/*
 *  TOPPERS/A-WDG
 *      Automotive Watchdog Driver
 *
 *  Copyright (C) 2014-2015 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2015 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2015 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2015 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2015 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2014-2015 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2015 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: Wdg.h 3492 2017-03-14 05:45:23Z suzuki-kawaguchi $
 */

/* [WDG061] Wdg.h */

#ifndef WDG_H
#define WDG_H

#include "Os.h"
#include "Wdg_Cfg.h"
#include "WdgIf_Types.h"

/*
 *  [WDG010] エラーコード
 */
#define WDG_E_DRIVER_STATE	0x10U
#define WDG_E_PARAM_MODE	0x11U
#define WDG_E_PARAM_CONFIG	0x12U
#define WDG_E_PARAM_TIMEOUT	0x13U
#define WDG_E_PARAM_POINTER	0x14U

/*
 *  タイムアウト後の動作
 */
#define WDG_RESET		0U
#define WDG_NMI			1U

#define WDG_INVALID_UINT16	(uint16) 0xffffU

/*
 *  [WDG152] モジュール内部状態
 */
#define WDG_UNINIT		0U
#define WDG_IDLE		1U
#define WDG_BUSY		2U

/*
 *  [WDG087] バージョン情報
 */
#define WDG_AR_RELEASE_MAJOR_VERSION		4U
#define WDG_AR_RELEASE_MINOR_VERSION		0U
#define WDG_AR_RELEASE_REVISION_VERSION		3U

/*
 *  サプライヤバージョン情報
 */
#define WDG_SW_MAJOR_VERSION				1U
#define WDG_SW_MINOR_VERSION				2U
#define WDG_SW_PATCH_VERSION				0U
#define WDG_VENDOR_ID						65U	/* NCESベンダID */

/*
 *  WdgモジュールID
 */
#define WDG_MODULE_ID						102U

/*
 *  [WDG063][WDG148] DETへ通知するサービスID
 */
#define WdgServiceId_Init					0x00U
#define WdgServiceId_SetMode				0x01U
#define WdgServiceId_SetTriggerCondition	0x03U
#define WdgServiceId_GetVersionInfo			0x04U

/*
 *  [WDG013][WDG045][WDG064][WDG065][WDG066][WDG111] DETエラーチェックマクロ
 */
#if WDG_DEV_ERROR_DETECT != FALSE
#define WDG_CHECK_INITIALIZED(ApiId)													\
	do {																				\
		if (wdg_state != WDG_IDLE) {													\
			(void) Det_ReportError(WDG_MODULE_ID, 0x00U, (ApiId), WDG_E_DRIVER_STATE);	\
			goto error_exit;															\
		}																				\
	} while (0)

#define WDG_CHECK_INITIALIZED_ERCD(ApiId)												\
	do {																				\
		if (wdg_state != WDG_IDLE) {													\
			ercd = E_NOT_OK;															\
			(void) Det_ReportError(WDG_MODULE_ID, 0x00U, (ApiId), WDG_E_DRIVER_STATE);	\
			goto error_exit;															\
		}																				\
	} while (0)

#define WDG_CHECK_PARAM(exp, ApiId, ErrorId)											\
	do {																				\
		if (!(exp)) {																	\
			(void) Det_ReportError(WDG_MODULE_ID, 0x00U, (ApiId), (ErrorId));			\
			goto error_exit;															\
		}																				\
	} while (0)

#define WDG_CHECK_PARAM_ERCD(exp, ApiId, ErrorId)										\
	do {																				\
		if (!(exp)) {																	\
			ercd = E_NOT_OK;															\
			(void) Det_ReportError(WDG_MODULE_ID, 0x00U, (ApiId), (ErrorId));			\
			goto error_exit;															\
		}																				\
	} while (0)

#define WDG_SET_STATE(state)															\
	do {																				\
		wdg_state = state;																\
	} while (0)
#else /* WDG_DEV_ERROR_DETECT == FALSE */
#define WDG_CHECK_INITIALIZED(ApiId)
#define WDG_CHECK_INITIALIZED_ERCD(ApiId)
#define WDG_CHECK_PARAM(exp, ApiId, ErrorId)
#define WDG_CHECK_PARAM_ERCD(exp, ApiId, ErrorId)
#define WDG_SET_STATE(state)
#endif /* WDG_DEV_ERROR_DETECT != FALSE */

/*
 *  [WDG110] DEMエラーチェックマクロ
 */
#define WDG_CHECK_DEM_ERCD(exp, EventId)												\
	do {																				\
		if ((!(exp)) && ((EventId) != WDG_INVALID_UINT16)) {							\
			ercd = E_NOT_OK;															\
			Dem_ReportErrorStatus((EventId), DEM_EVENT_STATUS_PASSED);					\
		}																				\
	} while (0)

#define WDG_CHECK_DEM_EXIT_ERCD(exp, EventId)											\
	do {																				\
		if ((!(exp)) && ((EventId) != WDG_INVALID_UINT16)) {							\
			ercd = E_NOT_OK;															\
			Dem_ReportErrorStatus((EventId), DEM_EVENT_STATUS_PASSED);					\
			goto error_exit;															\
		}																				\
	} while (0)

typedef uint8	Wdg_TimeoutReactionType;

/*
 *  トリガ管理ブロック
 */
typedef struct trigger_control_block {
	WdgIf_ModeType	current_mode;		/* [WDG154] 現在のウォッチドッグモード */
	uint16			trigger_cnt;		/* [WDG153]トリガ数 */
} TRIGGER_CB;

/*
 *  Wdgモード初期化ブロック
 */
typedef struct wdg_mode_block {
	Wdg_TimeoutReactionType			WdgTimeoutReaction;	/* タイムアウト後の動作 */
	uint8							WdgWindowOpenRate;	/* ウィンドウオープン期間 */
	uint16							timeout;			/* タイムアウト値 */
	uint16							interrupt_period;	/* 割り込み周期 */
} WDG_MODE_INIB;

/*
 *  [WDG171] コンフィギュレーション管理ブロック
 */
typedef struct wdg_config_type {
	WdgIf_ModeType		WdgDefaultMode;		/* デフォルトモード */
	const WDG_MODE_INIB	*p_fast_mode_inib;	/* WdgFastモード初期化ブロック */
	const WDG_MODE_INIB	*p_slow_mode_inib;	/* WdgSlowモード初期化ブロック */
	TRIGGER_CB			*p_trigger_cb;		/* トリガ管理ブロック */
} Wdg_ConfigType;

/*
 *  Wdgシステムサービスプロトタイプ宣言
 */
extern void Wdg_Init(const Wdg_ConfigType *ConfigPtr);
extern Std_ReturnType Wdg_SetMode(WdgIf_ModeType WdgMode);
extern void Wdg_SetTriggerCondition(uint16 timeout);
extern void Wdg_GetVersionInfo(Std_VersionInfoType *versioninfo);

/*
 *  Wdgコンフィギュレーション情報(Wdg_Irq.c)
 */
extern const Wdg_ConfigType	*p_cur_wdg_config;

/*
 *  Wdgモード初期化ブロック取得
 */
LOCAL_INLINE const WDG_MODE_INIB *
get_wdg_mode_inib(const WdgIf_ModeType mode)
{
	const WDG_MODE_INIB *current_mode;

	switch (mode) {
	case WDGIF_FAST_MODE:
		current_mode = p_cur_wdg_config->p_fast_mode_inib;
		break;
	case WDGIF_SLOW_MODE:
		current_mode = p_cur_wdg_config->p_slow_mode_inib;
		break;
	default:
		current_mode = NULL_PTR;
		break;
	}

	return(current_mode);
}

/*
 *  以下，未サポート機能
 */
extern void Wdg_Cbk_GptNotification(void);

/*
 *  未サポート
 *  - GPT
 *    <WDG163><WDG164><WDG165>
 *  - 複数ウォッチドッグ
 *    <WDG169><WDG170><WDG172>
 *  - 外部ウォッチドッグ
 *    <WDG055><WDG076><WDG162><WDG077><WDG078>
 *  - リンクタイム
 *    <WDG158>
 *  - デバッグ機能
 *    <WDG149><WDG150><WDG151>
 *  - Offモードコンテナ
 *    <WDG122_Conf>
 *  - 仕様ではない
 *    <WDG175>
 */

#endif /* WDG_H */
