/*
 *  TOPPERS/A-WDG
 *      Automotive Watchdog Driver
 *
 *  Copyright (C) 2014-2015 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2015 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2015 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2015 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2015 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2014-2015 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2015 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: Wdg_Target.h 3492 2017-03-14 05:45:23Z suzuki-kawaguchi $
 */

#ifndef WDG_TARGET_H
#define WDG_TARGET_H

#include "SchM_Wdg.h"
#include "prc_sil.h"
#include "Std_Types.h"
#include "Wdg.h"

#define TIMER_CLOCK				(uint32) 60000U

#ifdef TOPPERS_USE_MULTICORE
/* ウォッチドッグタイマのベースアドレス */
#define WDGT_BASE				0x08020400U
/* ウォッチドッグタイマトリガ割込み番号 */
#define WDGT_TRIGGER_IRQ		0x10012U
/* ウォッチドッグタイマトリガのベースアドレス */
#define WDGT_TRIGGER_BASE		(UINT_C(0x0c000200) + (UINT_C(0x00100000) * (((WDGT_TRIGGER_IRQ) >> UINT_C(16)) - UINT_C(1))) + (UINT_C(0x20) * (((WDGT_TRIGGER_IRQ) &UINT_C(0xffff)) - UINT_C(16))))
#else
/* ウォッチドッグタイマのベースアドレス */
#define WDGT_BASE				0x08000080U
/* ウォッチドッグタイマトリガ割込み番号 */
#define WDGT_TRIGGER_IRQ		30U
/* ウォッチドッグタイマトリガのベースアドレス */
#define WDGT_TRIGGER_BASE		(0x08000000U + (0x00000020U * WDGT_TRIGGER_IRQ))
#endif /* TOPPERS_USE_MULTICORE */

#define WDGT_STATUS				0x00U
#define WDGT_CONTROL			0x04U
#define WDGT_PERIODL			0x08U
#define WDGT_PERIODH			0x0CU
#define WDGT_SNAPL				0x10U
#define WDGT_SNAPH				0x14U

/* ビット定義 */
#define WDGT_STATUS_TO			0x01U
#define WDGT_STATUS_RUN			0x02U
#define WDGT_CONTROL_ITO		0x01U
#define WDGT_CONTROL_CONT		0x02U
#define WDGT_CONTROL_START		0x04U
#define WDGT_CONTROL_STOP		0x08U

#define SIL_WRW_IOP_WDGT_TRIGGER_BASE(mem, data)	sil_wrw_iop((void *) (WDGT_TRIGGER_BASE + (mem)), (data))
#define SIL_WRW_IOP_WDGT_BASE(mem, data)			sil_wrw_iop((void *) (WDGT_BASE + (mem)), (data))

/* 書込み用レジスタアクセスマクロ */
#define WDGT_WR_TRIGGER_CONTROL(data)				SIL_WRW_IOP_WDGT_TRIGGER_BASE(WDGT_CONTROL, data)
#define WDGT_WR_TRIGGER_STATUS(data)				SIL_WRW_IOP_WDGT_TRIGGER_BASE(WDGT_STATUS, data)
#define WDGT_WR_TRIGGER_PERIODL(data)				SIL_WRW_IOP_WDGT_TRIGGER_BASE(WDGT_PERIODL, data)
#define WDGT_WR_TRIGGER_PERIODH(data)				SIL_WRW_IOP_WDGT_TRIGGER_BASE(WDGT_PERIODH, data)
#define WDGT_WR_CONTROL(data)						SIL_WRW_IOP_WDGT_BASE(WDGT_CONTROL, data)
#define WDGT_WR_STATUS(data)						SIL_WRW_IOP_WDGT_BASE(WDGT_STATUS, data)
#define WDGT_WR_PERIODL(data)						SIL_WRW_IOP_WDGT_BASE(WDGT_PERIODL, data)
#define WDGT_WR_PERIODH(data)						SIL_WRW_IOP_WDGT_BASE(WDGT_PERIODH, data)

/* 内部関数プロトタイプ宣言 */
LOCAL_INLINE void wdg_trigger_init(uint16 wdg_interrupt_period);
LOCAL_INLINE void wdg_timer_init(const WDG_MODE_INIB *p_current_mode);
LOCAL_INLINE void wdg_trigger_terminate(void);
LOCAL_INLINE void wdg_trigger_int_clear(void);
LOCAL_INLINE void wdg_timer_terminate(void);
LOCAL_INLINE void wdg_timer_trigger(const WDG_MODE_INIB *p_current_mode);
LOCAL_INLINE void wdg_target_start(const WDG_MODE_INIB *p_current_mode);
LOCAL_INLINE void wdg_target_off_mode(void);
LOCAL_INLINE void wdg_target_set_mode(const WDG_MODE_INIB *p_current_mode);
LOCAL_INLINE uint16 wdg_target_reset_timeout(uint16 trigger_cnt, uint16 pre_interrupt_period);

/*
 *  ウォッチドッグタイマトリガ割込みの起動処理
 */
LOCAL_INLINE void
wdg_trigger_init(uint16 wdg_interrupt_period)
{
	/* タイマ停止 */
	WDGT_WR_TRIGGER_CONTROL(WDGT_CONTROL_STOP);

	/* タイムアウトステータスクリア */
	WDGT_WR_TRIGGER_STATUS(0x00U);

	/* タイマトリガ割込み周期設定 */
	/* カウンタ設定 下位16bit */
	WDGT_WR_TRIGGER_PERIODL((wdg_interrupt_period * TIMER_CLOCK) & 0xffffU);
	/* カウンタ設定 上位16bit */
	WDGT_WR_TRIGGER_PERIODH((wdg_interrupt_period * TIMER_CLOCK) >> 16U);

	/* タイマ開始，オートリロード，割込み許可 */
	WDGT_WR_TRIGGER_CONTROL(WDGT_CONTROL_START | WDGT_CONTROL_CONT | WDGT_CONTROL_ITO);

	return;
}

/*
 *  [WDG161] ウォッチドッグタイマ起動処理
 */
LOCAL_INLINE void
wdg_timer_init(const WDG_MODE_INIB *p_current_mode)
{
	/* タイマ停止 */
	WDGT_WR_CONTROL(WDGT_CONTROL_STOP);

	/* タイムアウトステータスクリア */
	WDGT_WR_STATUS(0x00U);

	/* タイマ周期設定 */
	/* カウンタ設定 下位16bit */
	WDGT_WR_PERIODL((p_current_mode->timeout * TIMER_CLOCK) & 0xffffU);
	/* カウンタ設定 上位16bit */
	WDGT_WR_PERIODH((p_current_mode->timeout * TIMER_CLOCK) >> 16U);

	/* タイマ開始 */
	WDGT_WR_CONTROL(WDGT_CONTROL_START);

	return;
}

/*
 *  ウォッチドッグタイマトリガ割込みの停止処理
 */
LOCAL_INLINE void
wdg_trigger_terminate(void)
{
	/* タイマ停止 */
	WDGT_WR_TRIGGER_CONTROL(WDGT_CONTROL_STOP);

	/* タイムアウトステータスクリア */
	WDGT_WR_TRIGGER_STATUS(0x00U);

	return;
}

/*
 *  ウォッチドッグタイマトリガ割込み要求クリア
 */
LOCAL_INLINE void
wdg_trigger_int_clear(void)
{
	/* タイムアウトステータスクリア */
	WDGT_WR_TRIGGER_STATUS(0x00U);

	return;
}

/*
 *  [WDG161] ウォッチドッグタイマ停止処理
 */
LOCAL_INLINE void
wdg_timer_terminate(void)
{
	/* タイマ停止 */
	WDGT_WR_CONTROL(WDGT_CONTROL_STOP);

	/* タイムアウトステータスクリア */
	WDGT_WR_STATUS(0x00U);

	return;
}

/*
 *  [WDG161] ウォッチドッグタイマトリガ処理
 */
LOCAL_INLINE void
wdg_timer_trigger(const WDG_MODE_INIB *p_current_mode)
{
	/* [WDG040] 割り込み無効 */
	SchM_Enter_Wdg_Reentrant();

	/* タイマ停止 */
	WDGT_WR_CONTROL(WDGT_CONTROL_STOP);

	/* タイマ周期設定 */
	/* カウンタ設定 下位16bit */
	WDGT_WR_PERIODL((p_current_mode->timeout * TIMER_CLOCK) & 0xffffU);
	/* カウンタ設定 上位16bit */
	WDGT_WR_PERIODH((p_current_mode->timeout * TIMER_CLOCK) >> 16U);

	/* タイマ開始 */
	WDGT_WR_CONTROL(WDGT_CONTROL_START);

	/* [WDG040] 割り込み有効 */
	SchM_Exit_Wdg_Reentrant();

	return;
}

/*
 *  ウォッチドッグ依存部初期化処理
 */
LOCAL_INLINE void
wdg_target_start(const WDG_MODE_INIB *p_current_mode)
{
	/* タイマ開始 */
	wdg_timer_init(p_current_mode);

	/* タイマトリガ割込み開始 */
	wdg_trigger_init(p_current_mode->interrupt_period);

	return;
}

/*
 *  ウォッチドッグ依存部offモード設定
 */
LOCAL_INLINE void
wdg_target_off_mode(void)
{
	/* タイマ停止 */
	wdg_timer_terminate();

	/* タイマトリガ割込み停止 */
	wdg_trigger_terminate();

	return;
}

/*
 *  ウォッチドッグ依存部モード設定
 */
LOCAL_INLINE void
wdg_target_set_mode(const WDG_MODE_INIB *p_current_mode)
{
	/* タイマ停止 */
	WDGT_WR_TRIGGER_CONTROL(WDGT_CONTROL_STOP);

	/* タイムアウトステータスクリア */
	WDGT_WR_TRIGGER_STATUS(0x00U);

	/* タイマトリガ割込み周期設定 */
	/* カウンタ設定 下位16bit */
	WDGT_WR_TRIGGER_PERIODL((p_current_mode->interrupt_period * TIMER_CLOCK) & 0xffffU);
	/* カウンタ設定 上位16bit */
	WDGT_WR_TRIGGER_PERIODH((p_current_mode->interrupt_period * TIMER_CLOCK) >> 16U);

	/* タイマ開始，オートリロード，割込み許可 */
	WDGT_WR_TRIGGER_CONTROL(WDGT_CONTROL_START | WDGT_CONTROL_CONT | WDGT_CONTROL_ITO);

	return;
}

/*
 *  タイムアウト再設定
 */
LOCAL_INLINE uint16
wdg_target_reset_timeout(uint16 trigger_cnt, uint16 pre_interrupt_period)
{
	uint16 timeout;

	if (trigger_cnt > 1U) {
		timeout = (trigger_cnt - 1) * pre_interrupt_period;
	}
	else if (trigger_cnt == 1U) {
		timeout = pre_interrupt_period;
	}
	else {
		timeout = 0U;
	}

	return(timeout);
}

/*
 *  未サポート
 *  - アクティベーションコード
 *    <WDG093><WDG094><WDG095>
 *  - トリガロケーション(メモリアドレス)設定
 *    <WDG034>
 *  - ランエリア(ROM or RAM)選択
 *    <WDG168>
 *  - 非依存部との分離のため
 *    <WDG031>
 */

#endif /* WDG_TARGET_H */
