/*
 *  TOPPERS/A-WDG
 *      Automotive Watchdog Driver
 *
 *  Copyright (C) 2014-2015 by Center for Embedded Computing Systems
 *                             Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2014-2015 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2014-2015 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2014-2015 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2014-2015 by SCSK Corporation, JAPAN
 *  Copyright (C) 2014-2015 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2014-2015 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2014-2015 by Witz Corporation
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: os.c 3492 2017-03-14 05:45:23Z suzuki-kawaguchi $
 */

#include "Os.h"
#include "Wdg.h"
#include "Wdg_PBcfg.h"
#include "t_syslog.h"
#include "prc_sil.h"

/* DE2-115 */
#define KEY_BASE	0x08000400U
#define LED_G_BASE	0x08000420U
#define KEY_0		0x01U
#define KEY_1		0x02U
#define KEY_2		0x04U
#define KEY_3		0x08U

TASK(MainTask)
{
	boolean	trigger = FALSE;
	uint8	ercd;
	uint32	cur_key, pre_key = 0xFFFFU;

	syslog(LOG_NOTICE, "=== Wdg_Init ===");
	Wdg_Init(&wdg_config_WdgSettingsConfig_0);

	while (1) {
		cur_key = sil_rew_iop((void *) KEY_BASE);

		/* ボタン押下状態に遷移した場合のみ処理を行う */
		/* SLOW/FASTモード切換え */
		if (((pre_key & KEY_0) != 0U) && ((cur_key & KEY_0) == 0U)) {

			if (trigger == FALSE) {
				syslog(LOG_NOTICE, "[KEY0] Wdg_SetMode(WDGIF_SLOW_MODE)");
				ercd = Wdg_SetMode(WDGIF_SLOW_MODE);
				trigger = TRUE;
			}
			else {
				syslog(LOG_NOTICE, "[KEY0] Wdg_SetMode(WDGIF_FAST_MODE)");
				ercd = Wdg_SetMode(WDGIF_FAST_MODE);
				trigger = FALSE;
			}
			/* LED1 */
			sil_wrw_iop((void *) LED_G_BASE, 1U);
		}
		/* OFFモード切換え */
		else if (((pre_key & KEY_1) != 0U) && ((cur_key & KEY_1) == 0U)) {
			syslog(LOG_NOTICE, "[KEY1] Wdg_SetMode(WDGIF_OFF_MODE)");
			ercd = Wdg_SetMode(WDGIF_OFF_MODE);
			/* LED2 */
			sil_wrw_iop((void *) LED_G_BASE, 3U);
		}
		/* トリガ設定 */
		else if (((pre_key & KEY_2) != 0U) && ((cur_key & KEY_2) == 0U)) {
			syslog(LOG_NOTICE, "[KEY2] Wdg_SetTriggerCondition(5000U)");
			Wdg_SetTriggerCondition(5000U);
			/* LED3 */
			sil_wrw_iop((void *) LED_G_BASE, 7U);
		}
		/* トリガ0設定 */
		else if (((pre_key & KEY_3) != 0U) && ((cur_key & KEY_3) == 0U)) {
			syslog(LOG_NOTICE, "[KEY3] Wdg_SetTriggerCondition(0U)");
			Wdg_SetTriggerCondition(0U);
			/* LED4 */
			sil_wrw_iop((void *) LED_G_BASE, 15U);
		}
		else {
			/* 何もしない */
		}

		pre_key = cur_key;
	}

	ShutdownOS(E_OK);
}

ProtectionReturnType
ProtectionHook(StatusType FatalError)
{
	syslog(LOG_NOTICE, "## ProtectionHook is called !! (%d)", FatalError);

	return(PRO_SHUTDOWN);
}

sint32
main(void)
{
	StartOS(MainApp);
	while (1) ;
	return(0);
}
